// Plot the displacements of m1 and m2 for 1SPP in the case of positive gap.

clear
clf();

//Given matrix M and K,and distance g
//m1 = 3.0; m2 = 5.0; k1 =6.0; k2 =10.0; // a family of parameters for another admissible 1SPP

m1 = 1.0; m2 = 1.0; k1 =1.0; k2 =10.0;
g = 1.0;

//---------------------------------------------------------
//Find matrix P such that P'MP =I and P'KP = diag(w1^2;w2^2)

M=[m1,0;0,m2];
K=[k1+k2,-k2;-k2,k2];
L=[sqrt(m1),0;0,sqrt(m2)];
Z=inv(L)*K*inv(L');
omega=spec(Z);
[R,diagevals] =spec(Z);

P=inv(L)*R;
iP=inv(P);
C=P'*K*P-diagevals*P'*M*P; // check that P satisfies these conditions <10^-15 is ok

// eigenvalues
w1=sqrt(diagevals(1,1));
w2=sqrt(diagevals(2,2));


//--------------------------------------------------------
//Define functions Phi1(s), Phi2(s)
function Phi1=Phi1(s)
    Phi1=sin(w1*s)/(w1*(1-cos(w1*s)))
endfunction

function Phi2=Phi2(s)
    Phi2=sin(w2*s)/(w2*(1-cos(w2*s)))
endfunction

//Define the equation w.r.t s, in order to find s where s=T-T1

function W1=W1(s)
    W1=-(P(1,1)*iP(1,1)*Phi1(s)+P(1,2)*iP(2,1)*Phi2(s))
endfunction

function W2=W2(s)
    W2=-(P(2,1)*iP(1,1)*Phi1(s)+P(2,2)*iP(2,1)*Phi2(s))
endfunction

function h=h(s)
    h=W1(s)-W2(s)
endfunction

// Find v (initial velocity of m1) wrt s
function v=v(s)
    v=g/W1(s)
endfunction

// Define the parameters in the formula of the sticking time T1
eta=(k1*g)/(k1+k2);

xi=-sqrt((k1+k2)*m1)/(g*k1);

omega0=sqrt((k1+k2)/m1);

// Define T1- the first time such that u1(T1)=g
//found from the fact that u1(T1) =g or A cos (omega0*T1+phi) = eta = A cos(phi)
// => cos(omega0*T1 +phi) =cos(phi) then we have the formula of T1
// T1 = (-2*phi+k*2%pi)/omega0

function T1=T1(v) 
    T1=-2*phi(v)/omega0 
endfunction 

function A=A(v) 
    A=eta/cos(phi(v)) 
endfunction 

function phi=phi(v) 
    phi=atan(xi*v) 
endfunction

// Define the linear 2dof system (~ system of 1st order ODEs)
function udot=sys2(t,u)
         udot=B*u
endfunction
B=[0,0,1,0;0,0,0,1;-(k1+k2)/m1,k2/m1,0,0;k2/m2,-k2/m2,0,0];

// Define the linear  1dof system during sticking phase 
function u1dot=sys1(t,u1)
    u1dot=P1*u1+C1
endfunction
P1=[0,1;-(k1+k2)/m1,0];
C1=[0;k2*g/m1];


//--------------------------------------------------------------
//Find the free flight s0
s0=fsolve(8,h);

// Find the initial velocity of m1
v=v(s0);

// Find the sticking time T1
T=T1(v)+s0;

//Solve system with time from t0 to T2, then draw the gragh of (t,u2)

t0=T1(v);u0=[g;g;-v;0];
t=T1(v):0.01:T;
usol=ode(u0,t0,t,sys2);

// Draw the position of the wall during free flight
plot(t',g,'gr-')

//Draw the time history of displacements after sticking pĥase
plot2d(t',[usol(1,:)' usol(2,:)'],style=[2,5]);
e=gce();
hl=captions(e.children,['mass 1';'mass 2'],'in_lower_left');

// Solve system during sticking phase where there is only the motion of mass m1
t10=0;u10=[g;v];
t=0:0.01:t0;
u1sol=ode(u10,t10,t,sys1);

//mass m2 stay at the wall during sticking time 0 -> T1
u2sol=g*ones(length(t),1);

//Draw the wall as a green line during sticking phase
plot(t',g,'gr-')

//draw the time history of displacements during sticking time 0 ->T1
plot2d(t',[u1sol(1,:)' u2sol],style=[11,5],rect=[0,-1.5,T,1.5]);
e=gce();
hl=captions(e.children,['mass 1';'mass 2'],'in_lower_left');
xtitle( '', 'time (t)', 'Displacements (u)') ;


